<?php
/*
Plugin Name: Conversion Rate (EasyToolsWP)
Description: A plugin to calculate and display conversion rates in the WooCommerce admin dashboard.
Version: 1.0.2
Author: EasyToolsWP.com
*/

// Forhindre direkte adgang
if (!defined('ABSPATH')) {
    exit;
}

// Tjek om WooCommerce er aktiveret
function eta_is_woocommerce_active() {
    return class_exists('WooCommerce');
}

// Opret custom database tabel
function eta_create_visitor_table() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'eta_visitors';
    $charset_collate = $wpdb->get_charset_collate();
    
    $sql = "CREATE TABLE $table_name (
        id BIGINT(20) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
        ip_address VARCHAR(100) NOT NULL,
        visit_date DATE NOT NULL,
        UNIQUE KEY unique_visit (ip_address, visit_date)
    ) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);
}
register_activation_hook(__FILE__, 'eta_create_visitor_table');

// Registrer besøgende
function eta_record_visitor() {
    if (!eta_is_woocommerce_active()) {
        return;
    }

    global $wpdb;
    $table_name = $wpdb->prefix . 'eta_visitors';
    
    $ip_address = $_SERVER['REMOTE_ADDR'];
    $visit_date = date('Y-m-d');
    
    $exists = $wpdb->get_var($wpdb->prepare(
        "SELECT COUNT(*) FROM $table_name WHERE ip_address = %s AND visit_date = %s",
        $ip_address, $visit_date
    ));
    
    if ($exists == 0) {
        $wpdb->insert(
            $table_name,
            [
                'ip_address' => $ip_address,
                'visit_date' => $visit_date
            ]
        );
    }
}
add_action('wp_head', 'eta_record_visitor');

// Hent antal salg fra WooCommerce
function eta_get_woocommerce_sales_count($start_date, $end_date) {
    if (!eta_is_woocommerce_active()) {
        return 0;
    }

    $args = [
        'post_type' => 'shop_order',
        'post_status' => 'wc-completed',
        'date_query' => [
            'after' => $start_date,
            'before' => $end_date,
            'inclusive' => true,
        ],
        'fields' => 'ids',
        'posts_per_page' => -1, // Get all orders
    ];

    $orders = get_posts($args);
    return count($orders);
}

// Hent antal besøgende
function eta_get_visitors_count($start_date, $end_date) {
    global $wpdb;
    $table_name = $wpdb->prefix . 'eta_visitors';

    $count = $wpdb->get_var($wpdb->prepare(
        "SELECT COUNT(DISTINCT ip_address) FROM $table_name WHERE visit_date BETWEEN %s AND %s",
        $start_date, $end_date
    ));
    
    return $count ? $count : 0;
}

// Beregn konverteringsrate og vis i widget
function eta_dashboard_widget_display() {
    if (!eta_is_woocommerce_active()) {
        echo '<p>WooCommerce is not activated. Please install and activate WooCommerce to use this plugin.</p>';
        return;
    }

    // Define predefined periods
    $periods = [
        'custom' => 'Custom Range',
        'today' => 'Today',
        'yesterday' => 'Yesterday',
        'this_week' => 'This Week (Since Monday)',
        'last_7_days' => 'Last 7 Days',
        'this_month' => 'This Month (Since 1st)',
        'last_30_days' => 'Last 30 Days',
        'last_60_days' => 'Last 60 Days',
        'last_90_days' => 'Last 90 Days',
        'last_180_days' => 'Last 180 Days',
        'this_year' => 'This Year (Since Jan. 1st)',
        'last_365_days' => 'Last 365 Days'
    ];

    // Get selected period
    $selected_period = isset($_POST['eta_period']) ? $_POST['eta_period'] : 'today';
    $start_date = isset($_POST['eta_start_date']) ? sanitize_text_field($_POST['eta_start_date']) : '';
    $end_date = isset($_POST['eta_end_date']) ? sanitize_text_field($_POST['eta_end_date']) : '';

    if ($selected_period === 'custom' && $start_date && $end_date) {
        if ($start_date > $end_date) {
            echo '<p style="color: red;">Start date cannot be later than end date.</p>';
            return;
        }
    } else {
        // Calculate start and end dates based on selected period
        switch ($selected_period) {
            case 'today':
                $start_date = $end_date = date('Y-m-d');
                break;
            case 'yesterday':
                $start_date = $end_date = date('Y-m-d', strtotime('-1 day'));
                break;
            case 'this_week':
                $start_date = date('Y-m-d', strtotime('monday this week'));
                $end_date = date('Y-m-d');
                break;
            case 'last_7_days':
                $start_date = date('Y-m-d', strtotime('-6 days'));
                $end_date = date('Y-m-d');
                break;
            case 'this_month':
                $start_date = date('Y-m-01');
                $end_date = date('Y-m-d');
                break;
            case 'last_30_days':
                $start_date = date('Y-m-d', strtotime('-29 days'));
                $end_date = date('Y-m-d');
                break;
            case 'last_60_days':
                $start_date = date('Y-m-d', strtotime('-59 days'));
                $end_date = date('Y-m-d');
                break;
            case 'last_90_days':
                $start_date = date('Y-m-d', strtotime('-89 days'));
                $end_date = date('Y-m-d');
                break;
            case 'last_180_days':
                $start_date = date('Y-m-d', strtotime('-179 days'));
                $end_date = date('Y-m-d');
                break;
            case 'this_year':
                $start_date = date('Y-01-01');
                $end_date = date('Y-m-d');
                break;
            case 'last_365_days':
                $start_date = date('Y-m-d', strtotime('-364 days'));
                $end_date = date('Y-m-d');
                break;
            default:
                $start_date = $end_date = date('Y-m-d');
                break;
        }
    }

    // Comparison period for the same duration in the previous year
    $comparison_start_date = date('Y-m-d', strtotime($start_date . ' -1 year'));
    $comparison_end_date = date('Y-m-d', strtotime($end_date . ' -1 year'));

    // Get data for current period
    $sales_count = eta_get_woocommerce_sales_count($start_date, $end_date);
    $visitors_count = eta_get_visitors_count($start_date, $end_date);
    $conversion_rate = $visitors_count > 0 ? ($sales_count / $visitors_count) * 100 : 0;

    // Get data for comparison period
    $comparison_sales_count = eta_get_woocommerce_sales_count($comparison_start_date, $comparison_end_date);
    $comparison_visitors_count = eta_get_visitors_count($comparison_start_date, $comparison_end_date);
    $comparison_conversion_rate = $comparison_visitors_count > 0 ? ($comparison_sales_count / $comparison_visitors_count) * 100 : 0;

    // Calculate percentage change
    $sales_count_change = $comparison_sales_count > 0 ? (($sales_count - $comparison_sales_count) / $comparison_sales_count) * 100 : 0;
    $visitors_count_change = $comparison_visitors_count > 0 ? (($visitors_count - $comparison_visitors_count) / $comparison_visitors_count) * 100 : 0;
    $conversion_rate_change = $comparison_conversion_rate > 0 ? (($conversion_rate - $comparison_conversion_rate) / $comparison_conversion_rate) * 100 : 0;

    ?>
    <form method="post" style="margin-bottom: 20px;">
        <label for="eta_period">Select Period:</label>
        <select name="eta_period" id="eta_period" onchange="this.form.submit()">
            <?php foreach ($periods as $value => $label) : ?>
                <option value="<?php echo esc_attr($value); ?>" <?php selected($selected_period, $value); ?>><?php echo esc_html($label); ?></option>
            <?php endforeach; ?>
        </select>

        <div id="eta_custom_date_picker" style="display: <?php echo $selected_period === 'custom' ? 'block' : 'none'; ?>; margin-top: 10px;">
            <label for="eta_start_date">Start Date:</label>
            <input type="date" name="eta_start_date" id="eta_start_date" value="<?php echo esc_attr($start_date); ?>">
            
            <label for="eta_end_date">End Date:</label>
            <input type="date" name="eta_end_date" id="eta_end_date" value="<?php echo esc_attr($end_date); ?>">
            
            <input type="submit" value="Update">
        </div>
    </form>

    <style>
        .eta-dashboard-widget {
            display: flex;
            justify-content: space-between;
        }
        .eta-dashboard-widget .eta-box {
            flex: 1;
            margin-right: 10px;
            padding: 15px;
            background-color: #f1f1f1;
            border-radius: 8px;
            text-align: center;
            box-shadow: 0 2px 4px rgba(0, 0, 0, 0.1);
        }
        .eta-dashboard-widget .eta-box:last-child {
            margin-right: 0;
        }
        .eta-dashboard-widget .eta-box h4 {
            margin: 0 0 10px;
            font-size: 16px;
            color: #555;
        }
        .eta-dashboard-widget .eta-box p {
            margin: 0;
            font-size: 24px;
            font-weight: bold;
            color: #333;
        }
        .eta-dashboard-widget .eta-box small {
            display: block;
            margin-top: 5px;
            font-size: 14px;
            color: #888;
        }
        .eta-dashboard-widget .eta-box small .eta-change-positive {
            color: green;
        }
        .eta-dashboard-widget .eta-box small .eta-change-negative {
            color: red;
        }
    </style>
    <div class="eta-dashboard-widget">
        <div class="eta-box">
            <h4>Conversion Rate</h4>
            <p><?php echo number_format($conversion_rate, 2); ?>%</p>
            <small>Last year: <?php echo number_format($comparison_conversion_rate, 2); ?>% 
                <br /><span class="<?php echo $conversion_rate_change >= 0 ? 'eta-change-positive' : 'eta-change-negative'; ?>">
                <?php echo number_format($conversion_rate_change, 2); ?>%</span>
            </small>
        </div>
        <div class="eta-box">
            <h4>Total Visitors</h4>
            <p><?php echo esc_html($visitors_count); ?></p>
            <small>Last year: <?php echo esc_html($comparison_visitors_count); ?> 
                <br /><span class="<?php echo $visitors_count_change >= 0 ? 'eta-change-positive' : 'eta-change-negative'; ?>">
                <?php echo number_format($visitors_count_change, 2); ?>%</span>
            </small>
        </div>
        <div class="eta-box">
            <h4>Total Sales</h4>
            <p><?php echo esc_html($sales_count); ?></p>
            <small>Last year: <?php echo esc_html($comparison_sales_count); ?> 
                <br /><span class="<?php echo $sales_count_change >= 0 ? 'eta-change-positive' : 'eta-change-negative'; ?>">
                <?php echo number_format($sales_count_change, 2); ?>%</span>
            </small>
        </div>
    </div>

    <script>
        document.getElementById('eta_period').addEventListener('change', function() {
            document.getElementById('eta_custom_date_picker').style.display = this.value === 'custom' ? 'block' : 'none';
        });
    </script>
    <?php
}

// Tilføj widget til dashboard
function eta_add_dashboard_widget() {
    wp_add_dashboard_widget(
        'eta_dashboard_widget',
        'ETA Conversion Rate',
        'eta_dashboard_widget_display'
    );
}
add_action('wp_dashboard_setup', 'eta_add_dashboard_widget');
?>
